
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkBinaryThresholdImageFilter.h"


int main( int argc, char * argv[] )
{
  if( argc < 5 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  outputImageFile  dimension lower_threshold [upper_threshold] [inside_value] [outside_value]" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;

  typedef unsigned char   InputPixelType;
  typedef unsigned char   OutputPixelType;
  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    const unsigned int Dimension = 2;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::BinaryThresholdImageFilter<InputImageType,InputImageType>ThresholdFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ThresholdFilterType::Pointer thresholder = ThresholdFilterType::New();
    thresholder->SetInput(reader->GetOutput());
    if (argc > 7)
      thresholder->SetOutsideValue((unsigned char)atoi(argv[7]));
    else
      thresholder->SetOutsideValue( 0 );
    if (argc > 6)
      thresholder->SetInsideValue((unsigned char)atoi(argv[6]));
    else
      thresholder->SetInsideValue(  255 );
    thresholder->SetLowerThreshold((unsigned char)atoi(argv[4]));
    if (argc > 5)
      thresholder->SetUpperThreshold((unsigned char)atoi(argv[5]));
    else
      thresholder->SetUpperThreshold( 255 );

    try {
      thresholder->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    writer->SetInput(thresholder->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    const unsigned int Dimension = 3;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::BinaryThresholdImageFilter<InputImageType,InputImageType>ThresholdFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ThresholdFilterType::Pointer thresholder = ThresholdFilterType::New();
    thresholder->SetInput(reader->GetOutput());
    if (argc > 7)
      thresholder->SetOutsideValue((unsigned char)atoi(argv[7]));
    else
      thresholder->SetOutsideValue( 0 );
    if (argc > 6)
      thresholder->SetInsideValue((unsigned char)atoi(argv[6]));
    else
      thresholder->SetInsideValue(  255 );
    thresholder->SetLowerThreshold((unsigned char)atoi(argv[4]));
    if (argc > 5)
      thresholder->SetUpperThreshold((unsigned char)atoi(argv[5]));
    else
      thresholder->SetUpperThreshold( 255 );

    try {
      thresholder->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    writer->SetInput(thresholder->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}
