
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkDanielssonDistanceMapImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"


int main( int argc, char * argv[] )
{
  if( argc < 4 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile outputImageFile dimension" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;
  
  typedef unsigned char   InputPixelType;
  typedef unsigned short  OutputPixelType;
  typedef unsigned char   WritePixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    const unsigned int Dimension = 2;
    typedef itk::Image< InputPixelType,  Dimension > InputImageType;
    typedef itk::Image< OutputPixelType, Dimension > OutputImageType;
    typedef itk::Image< WritePixelType,  Dimension > WriteImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< WriteImageType  >  WriterType;
    typedef itk::DanielssonDistanceMapImageFilter<InputImageType,OutputImageType>FilterType;
    typedef itk::RescaleIntensityImageFilter<OutputImageType,WriteImageType>RescalerType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    FilterType::Pointer distanceMap = FilterType::New();
    distanceMap->SetInput(  reader->GetOutput()     );
    distanceMap->InputIsBinaryOn();
    try {
      distanceMap->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(   0 );
    rescaler->SetOutputMaximum( 255 );
    rescaler->SetInput(  distanceMap->GetOutput()   );
    writer->SetInput(    rescaler->GetOutput()   );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    const unsigned int Dimension = 3;
    typedef itk::Image< InputPixelType,  Dimension > InputImageType;
    typedef itk::Image< OutputPixelType, Dimension > OutputImageType;
    typedef itk::Image< WritePixelType,  Dimension > WriteImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< WriteImageType  >  WriterType;
    typedef itk::DanielssonDistanceMapImageFilter<InputImageType,OutputImageType>FilterType;
    typedef itk::RescaleIntensityImageFilter<OutputImageType,WriteImageType>RescalerType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    FilterType::Pointer distanceMap = FilterType::New();
    distanceMap->SetInput(  reader->GetOutput()     );
    distanceMap->InputIsBinaryOn();
    try {
      distanceMap->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(   0 );
    rescaler->SetOutputMaximum( 255 );
    rescaler->SetInput(  distanceMap->GetOutput()   );
    writer->SetInput(    rescaler->GetOutput()   );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}
