
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkGradientAnisotropicDiffusionImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"

int main( int argc, char * argv[] )
{
  if( argc < 7 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  outputImageFile dimension iterations timeStep  conductance " << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;
  
  typedef float           InputPixelType;
  typedef float           InternalPixelType;
  typedef unsigned char   WritePixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    const unsigned int Dimension = 2;
    typedef itk::Image< InputPixelType,    Dimension >   InputImageType;
    typedef itk::Image< InternalPixelType, Dimension >   InternalImageType;
    typedef itk::Image< WritePixelType,    Dimension >   WriteImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< WriteImageType  >  WriterType;
    typedef itk::GradientAnisotropicDiffusionImageFilter<InputImageType,InternalImageType>FilterType;
    typedef itk::RescaleIntensityImageFilter<InternalImageType,WriteImageType>RescalerType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    FilterType::Pointer smoother = FilterType::New();
    smoother->SetInput(reader->GetOutput());
    smoother->SetNumberOfIterations(atoi( argv[4]));
    smoother->SetTimeStep((double) atof( argv[5]));
    smoother->SetConductanceParameter((double)atof(argv[6]));
    try {
      smoother->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(   0 );
    rescaler->SetOutputMaximum( 255 );
    rescaler->SetInput(smoother->GetOutput());
    writer->SetInput(rescaler->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    const unsigned int Dimension = 3;
    typedef itk::Image< InputPixelType,    Dimension >   InputImageType;
    typedef itk::Image< InternalPixelType, Dimension >   InternalImageType;
    typedef itk::Image< WritePixelType,    Dimension >   WriteImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< WriteImageType  >  WriterType;
    typedef itk::GradientAnisotropicDiffusionImageFilter<InputImageType,InternalImageType>FilterType;
    typedef itk::RescaleIntensityImageFilter<InternalImageType,WriteImageType>RescalerType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    FilterType::Pointer smoother = FilterType::New();
    smoother->SetInput(reader->GetOutput());
    smoother->SetNumberOfIterations(atoi( argv[4]));
    smoother->SetTimeStep((double) atof( argv[5]));
    smoother->SetConductanceParameter((double)atof(argv[6]));
    try {
      smoother->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    RescalerType::Pointer rescaler = RescalerType::New();
    rescaler->SetOutputMinimum(   0 );
    rescaler->SetOutputMaximum( 255 );
    rescaler->SetInput(smoother->GetOutput());
    writer->SetInput(rescaler->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  
  return 0;
}
