
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkMedianImageFilter.h"


int main( int argc, char * argv[] )
{
  if( argc < 4 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile outputImageFile dimension" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;
  //printf("the dimension for the file is %d\n",dim);

  typedef unsigned char   InputPixelType;
  typedef unsigned char   OutputPixelType;
  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    const unsigned int Dimension = 2;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::MedianImageFilter< InputImageType, 
      OutputImageType > MedianFilterType;
    InputImageType::SizeType indexRadius;
    indexRadius[0] = 1;
    indexRadius[1] = 1;
    
    //
    // Creation of Reader and Writer filters
    //
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    
    MedianFilterType::Pointer median_filter = MedianFilterType::New();
    median_filter->SetRadius(indexRadius);
    median_filter->SetInput(reader->GetOutput());

    try {
      median_filter->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }

    writer->SetInput(median_filter->GetOutput());

    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  else {
    const unsigned int Dimension = 3;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::MedianImageFilter< InputImageType, 
      OutputImageType > MedianFilterType;
    InputImageType::SizeType indexRadius;
    indexRadius[0] = 1;
    indexRadius[1] = 1;
    indexRadius[2] = 1;
    
    //
    // Creation of Reader and Writer filters
    //
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    
    MedianFilterType::Pointer median_filter = MedianFilterType::New();
    median_filter->SetRadius(indexRadius);
    median_filter->SetInput(reader->GetOutput());

    try {
      median_filter->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }

    writer->SetInput(median_filter->GetOutput());

    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}
