#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkRecursiveGaussianImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"

int main( int argc, char * argv[] )
{
  if( argc < 4 )  { 
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  outputImageFile dimension sigma" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;

  typedef float InputPixelType;
  typedef float OutputPixelType;
  typedef unsigned char WritePixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    typedef itk::Image< InputPixelType,  2 >   InputImageType;
    typedef itk::Image< OutputPixelType, 2 >   OutputImageType;
    typedef itk::Image< WritePixelType, 2 >   WriteImageType;
    typedef itk::ImageFileReader< InputImageType >  ReaderType;
    ReaderType::Pointer reader = ReaderType::New();
    reader->SetFileName( argv[1] );
    typedef itk::RecursiveGaussianImageFilter< InputImageType, 
      OutputImageType > FilterType;
    FilterType::Pointer filterX = FilterType::New();
    FilterType::Pointer filterY = FilterType::New();
    filterX->SetDirection(0); // x direction
    filterY->SetDirection(1); // y direction
    filterX->SetOrder(FilterType::ZeroOrder);
    filterY->SetOrder(FilterType::ZeroOrder);
    filterX->SetNormalizeAcrossScale(false);
    filterY->SetNormalizeAcrossScale(false);
    filterX->SetInput( reader->GetOutput() );
    filterY->SetInput( filterX->GetOutput() );
    filterX->SetSigma((double)atof(argv[3]));
    filterY->SetSigma((double)atof(argv[3]));
    
    try {
      filterY->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    typedef itk::RescaleIntensityImageFilter< 
      OutputImageType, WriteImageType > RescaleFilterType;
    RescaleFilterType::Pointer rescaler = RescaleFilterType::New();
    rescaler->SetOutputMinimum(   0 );
    rescaler->SetOutputMaximum( 255 );
      rescaler->SetInput( filterY->GetOutput() );
    typedef itk::ImageFileWriter< WriteImageType >  WriterType;
    WriterType::Pointer writer = WriterType::New();
    writer->SetFileName( argv[2] );
    writer->SetInput( rescaler->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    typedef itk::Image< InputPixelType,  3 >   InputImageType;
    typedef itk::Image< OutputPixelType, 3 >   OutputImageType;
    typedef itk::Image< WritePixelType, 3 >   WriteImageType;
    typedef itk::ImageFileReader< InputImageType >  ReaderType;
    ReaderType::Pointer reader = ReaderType::New();
    reader->SetFileName( argv[1] );
    typedef itk::RecursiveGaussianImageFilter< InputImageType, 
      OutputImageType > FilterType;
    FilterType::Pointer filterX = FilterType::New();
    FilterType::Pointer filterY = FilterType::New();
    filterX->SetDirection(0); // x direction
    filterY->SetDirection(1); // y direction
    filterX->SetOrder(FilterType::ZeroOrder);
    filterY->SetOrder(FilterType::ZeroOrder);
    filterX->SetNormalizeAcrossScale(false);
    filterY->SetNormalizeAcrossScale(false);
    filterX->SetInput( reader->GetOutput() );
    filterY->SetInput( filterX->GetOutput() );
    filterX->SetSigma((double)atof(argv[3]));
    filterY->SetSigma((double)atof(argv[3]));
    
    try {
      filterY->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    typedef itk::RescaleIntensityImageFilter< 
      OutputImageType, WriteImageType > RescaleFilterType;
    RescaleFilterType::Pointer rescaler = RescaleFilterType::New();
    rescaler->SetOutputMinimum(   0 );
    rescaler->SetOutputMaximum( 255 );
      rescaler->SetInput( filterY->GetOutput() );
    typedef itk::ImageFileWriter< WriteImageType >  WriterType;
    WriterType::Pointer writer = WriterType::New();
    writer->SetFileName( argv[2] );
    writer->SetInput( rescaler->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}


