
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkResampleImageFilter.h"
#include "itkAffineTransform.h"
#include "itkNearestNeighborInterpolateImageFunction.h"


int main( int argc, char * argv[] )
{
  if( argc < 4 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile outputImageFile dimension" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;

  typedef unsigned char   InputPixelType;
  typedef unsigned char   OutputPixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    const unsigned int Dimension = 2;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::ResampleImageFilter< InputImageType, 
      OutputImageType > ResampleFilterType;
    typedef itk::AffineTransform< double, Dimension > TransformType;
    typedef itk::NearestNeighborInterpolateImageFunction< InputImageType, double > InterpolatorType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ResampleFilterType::Pointer resample = ResampleFilterType::New();
    TransformType::Pointer transform = TransformType::New();
    resample->SetTransform(transform);
    InterpolatorType::Pointer interpolator = InterpolatorType::New();
    resample->SetInterpolator(interpolator);
    resample->SetDefaultPixelValue(0);  
    double spacing[2];
    spacing[0] = 1.0; // pixel spacing in mm along X
    spacing[1] = 1.0; // pixel spacing in mm along Y
    resample->SetOutputSpacing(spacing);
    double origin[2];
    origin[0] = 1.0; // X space coordinate of origin
    origin[1] = 1.0; // Y space coordinate of origin
    resample->SetOutputOrigin(origin);   
    InputImageType::SizeType size;
    size[0] = 300; // number of pixels along X
    size[1] = 300; // number of pixels along Y
    resample->SetSize(size);  
    resample->SetInput(reader->GetOutput());
    try {
      resample->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    writer->SetInput(resample->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    const unsigned int Dimension = 3;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::ResampleImageFilter< InputImageType, 
      OutputImageType > ResampleFilterType;
    typedef itk::AffineTransform< double, Dimension > TransformType;
    typedef itk::NearestNeighborInterpolateImageFunction< InputImageType, double > InterpolatorType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ResampleFilterType::Pointer resample = ResampleFilterType::New();
    TransformType::Pointer transform = TransformType::New();
    resample->SetTransform(transform);
    InterpolatorType::Pointer interpolator = InterpolatorType::New();
    resample->SetInterpolator(interpolator);
    resample->SetDefaultPixelValue(0);  
    double spacing[3];
    spacing[0] = 1.0; // pixel spacing in mm along X
    spacing[1] = 1.0; // pixel spacing in mm along Y
    spacing[2] = 1.0; // pixel spacing in mm along Z
    resample->SetOutputSpacing(spacing);
    double origin[3];
    origin[0] = 1.0; // X space coordinate of origin
    origin[1] = 1.0; // Y space coordinate of origin
    origin[2] = 1.0; // Z space coordinate of origin
    resample->SetOutputOrigin(origin);   
    InputImageType::SizeType size;
    size[0] = 300; // number of pixels along X
    size[1] = 300; // number of pixels along Y
    size[2] = 300; // number of pixels along Z
    resample->SetSize(size);  
    resample->SetInput(reader->GetOutput());
    try {
      resample->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    writer->SetInput(resample->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}




