
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkCastImageFilter.h"


int main( int argc, char * argv[] )
{
  if( argc < 5 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  outputImageFile dimension min_value max_value" << std::endl;
    return 1;
  }
  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;
  printf("dimension is: %d\n",dim);

  typedef unsigned char   InputPixelType;
  typedef float  OutputPixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    const unsigned int Dimension = 2;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< InputImageType  >  WriterType;
    typedef itk::RescaleIntensityImageFilter< 
      InputImageType, OutputImageType  >  RescaleIntensityFilterType;
    typedef itk::CastImageFilter< 
      OutputImageType, InputImageType  >  CastFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    RescaleIntensityFilterType::Pointer rescaler = RescaleIntensityFilterType::New(); 
    rescaler->SetInput(reader->GetOutput());
    rescaler->SetOutputMinimum((unsigned char)atoi(argv[4]));
    rescaler->SetOutputMaximum((unsigned char)atoi(argv[5]));
    try {
      rescaler->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    
    CastFilterType::Pointer castFilter = CastFilterType::New();
    castFilter->SetInput(rescaler->GetOutput());
    writer->SetInput(castFilter->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    const unsigned int Dimension = 3;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< InputImageType  >  WriterType;
    typedef itk::RescaleIntensityImageFilter< 
      InputImageType, OutputImageType  >  RescaleIntensityFilterType;
    typedef itk::CastImageFilter< 
      OutputImageType, InputImageType  >  CastFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    RescaleIntensityFilterType::Pointer rescaler = RescaleIntensityFilterType::New(); 
    rescaler->SetInput(reader->GetOutput());
    rescaler->SetOutputMinimum((unsigned char)atoi(argv[4]));
    rescaler->SetOutputMaximum((unsigned char)atoi(argv[5]));
    try {
      rescaler->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    
    CastFilterType::Pointer castFilter = CastFilterType::New();
    castFilter->SetInput(rescaler->GetOutput());
    writer->SetInput(castFilter->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}



