#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkShiftScaleImageFilter.h"
#include "itkCastImageFilter.h"


int main( int argc, char * argv[] )
{
  if( argc < 6 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  outputImageFile dimension scale_value shift_value" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;

  typedef unsigned char   InputPixelType;
  typedef float  OutputPixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    const unsigned int Dimension = 2;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< InputImageType  >  WriterType;
    typedef itk::ShiftScaleImageFilter< 
      InputImageType, OutputImageType  >  ShiftScaleFilterType;
    typedef itk::CastImageFilter< 
      OutputImageType, InputImageType  >  CastFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ShiftScaleFilterType::Pointer shiftscaler = ShiftScaleFilterType::New();
    shiftscaler->SetScale(atof(argv[4]));
    shiftscaler->SetShift(atof(argv[5]));
    shiftscaler->SetInput(reader->GetOutput());
    try {
      shiftscaler->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    
    CastFilterType::Pointer castFilter = CastFilterType::New();
    castFilter->SetInput(shiftscaler->GetOutput());
    writer->SetInput( castFilter->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    const unsigned int Dimension = 3;
    typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
    typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< InputImageType  >  WriterType;
    typedef itk::ShiftScaleImageFilter< 
      InputImageType, OutputImageType  >  ShiftScaleFilterType;
    typedef itk::CastImageFilter< 
      OutputImageType, InputImageType  >  CastFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ShiftScaleFilterType::Pointer shiftscaler = ShiftScaleFilterType::New();
    shiftscaler->SetScale(atof(argv[4]));
    shiftscaler->SetShift(atof(argv[5]));    
    shiftscaler->SetInput(reader->GetOutput());
    try {
      shiftscaler->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    
    CastFilterType::Pointer castFilter = CastFilterType::New();
    castFilter->SetInput(shiftscaler->GetOutput());
    writer->SetInput( castFilter->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}



