#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkSigmoidImageFilter.h"

int main( int argc, char * argv[] )
{
  if( argc < 7 )  { 
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile outputImageFile dimension sigmoidAlpha sigmoidBeta outputMinimum outputMaximum" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;

  typedef unsigned char InputPixelType;
  typedef unsigned char OutputPixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    typedef itk::Image< InputPixelType,  2 >   InputImageType;
    typedef itk::Image< OutputPixelType, 2 >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType >  ReaderType;
    ReaderType::Pointer reader = ReaderType::New();
    reader->SetFileName( argv[1] );
    typedef itk::SigmoidImageFilter< InputImageType, 
      OutputImageType > FilterType;
    FilterType::Pointer filter = FilterType::New();
    filter->SetInput( reader->GetOutput() );
    filter->SetOutputMinimum(atoi(argv[6]));
    filter->SetOutputMaximum(atoi(argv[7]));
    filter->SetAlpha(atof(argv[4]));
    filter->SetBeta(atof(argv[5]));
    try {
      filter->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    typedef unsigned char WritePixelType;
    typedef itk::Image< WritePixelType, 2 > WriteImageType;
    typedef itk::ImageFileWriter< WriteImageType >  WriterType;
    WriterType::Pointer writer = WriterType::New();
    writer->SetFileName( argv[2] );
    writer->SetInput( filter->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    typedef itk::Image< InputPixelType,  3 >   InputImageType;
    typedef itk::Image< OutputPixelType, 3 >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType >  ReaderType;
    ReaderType::Pointer reader = ReaderType::New();
    reader->SetFileName( argv[1] );
    typedef itk::SigmoidImageFilter< InputImageType, 
      OutputImageType > FilterType;
    FilterType::Pointer filter = FilterType::New();
    filter->SetInput( reader->GetOutput() );
    filter->SetOutputMinimum(atoi(argv[6]));
    filter->SetOutputMaximum(atoi(argv[7]));
    filter->SetAlpha(atof(argv[4]));
    filter->SetBeta(atof(argv[5]));
    try {
      filter->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    typedef unsigned char WritePixelType;
    typedef itk::Image< WritePixelType, 3 > WriteImageType;
    typedef itk::ImageFileWriter< WriteImageType >  WriterType;
    WriterType::Pointer writer = WriterType::New();
    writer->SetFileName( argv[2] );
    writer->SetInput( filter->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}
