
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkThresholdImageFilter.h"


int main( int argc, char * argv[] )
{
  if( argc < 7 ) {
    //std::cerr << "Usage: " << std::endl;
    //std::cerr << argv[0] << "  inputImageFile outputImageFile dimension method# (1,2,or3) outside_value lower_threshold [upper_threshold]" << std::endl;
    return 1;
  }

  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;
  printf("dimension: %d\n",dim);

  int methodNumber = 1;
  // get method type
  methodNumber = atoi(argv[4]);
  if (methodNumber < 0 || methodNumber > 3)
    methodNumber = 1;
  std::cerr << "Method number is: " << methodNumber << std::endl;

  // all methods require an outside value
  double outValue = (double)(atof(argv[5]));

  // all methods also require the lower threshold value
  double tmin = (double)(atof(argv[6]));

  double tmax = 255.0;		 
  if (argc > 7)
     tmax = (double)(atof(argv[7]));

  typedef unsigned char   InputPixelType;
  typedef unsigned char   OutputPixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    typedef itk::Image< InputPixelType,  2 >   InputImageType;
    typedef itk::Image< OutputPixelType, 2 >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::ThresholdImageFilter<InputImageType > ThresholdFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ThresholdFilterType::Pointer thresholder = ThresholdFilterType::New();
    thresholder->SetInput(reader->GetOutput());
    thresholder->SetOutsideValue(outValue);

    // Method 1: set threshold - change what is below
    if (methodNumber == 1)
      thresholder->ThresholdBelow(tmin);
    else if (methodNumber == 2)
      thresholder->ThresholdAbove(tmin);
    else
      thresholder->ThresholdOutside(tmin,tmax);
    try {
      thresholder->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    thresholder->SetInput(reader->GetOutput());
    writer->SetInput(thresholder->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    typedef itk::Image< InputPixelType,  3 >   InputImageType;
    typedef itk::Image< OutputPixelType, 3 >   OutputImageType;
    typedef itk::ImageFileReader< InputImageType  >  ReaderType;
    typedef itk::ImageFileWriter< OutputImageType >  WriterType;
    typedef itk::ThresholdImageFilter<InputImageType > ThresholdFilterType;
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    ThresholdFilterType::Pointer thresholder = ThresholdFilterType::New();
    thresholder->SetInput(reader->GetOutput());
    thresholder->SetOutsideValue(outValue);

    // Method 1: set threshold - change what is below
    if (methodNumber == 1)
      thresholder->ThresholdBelow(tmin);
    else if (methodNumber == 2)
      thresholder->ThresholdAbove(tmin);
    else
      thresholder->ThresholdOutside(tmin,tmax);
    try {
      thresholder->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    thresholder->SetInput(reader->GetOutput());
    writer->SetInput(thresholder->GetOutput());
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  return 0;
}
