
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkVectorGradientAnisotropicDiffusionImageFilter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkVectorCastImageFilter.h"
#include "itkRGBPixel.h"


int main( int argc, char * argv[] )
{
  if( argc < 6 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile outputImageFile dimension iterations timeStep" << std::endl;
    return 1;
  }
  
  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;
 
  typedef itk::VectorPixel<float>  InputPixelType;
  typedef float VectorPixelType;

  // define all of the classes based on the dimension that was input
  if (dim == 2) {
    typedef itk::Image< InputPixelType,2 > InputImageType;
    typedef itk::ImageFileReader< InputImageType > ReaderType;
    ReaderType::Pointer reader = ReaderType::New();
    reader->SetFileName( argv[1] );
    typedef itk::VectorGradientAnisotropicDiffusionImageFilter< 
      InputImageType,InputImageType > FilterType;
    FilterType::Pointer smoother = FilterType::New();
    smoother->SetInput(reader->GetOutput());
    smoother->SetNumberOfIterations(   atoi( argv[4] ) );
    smoother->SetTimeStep( atof( argv[5] ) );
    printf("iterations: %d timestep: %f\n",atoi( argv[4] ),atof( argv[5] ));
    try {
      smoother->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    typedef itk::VectorPixel<unsigned char> WritePixelType;
    typedef itk::Image<WritePixelType,2> WriteImageType;
    typedef itk::VectorCastImageFilter<InputImageType,WriteImageType>CasterType;
    CasterType::Pointer caster = CasterType::New();
    typedef itk::ImageFileWriter<WriteImageType> WriterType;
    WriterType::Pointer writer = WriterType::New();
    caster->SetInput(smoother->GetOutput());
    writer->SetInput(caster->GetOutput());
    writer->SetFileName( argv[2] );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  else {
    typedef itk::Image< InputPixelType,3 > InputImageType;
    typedef itk::ImageFileReader< InputImageType > ReaderType;
    ReaderType::Pointer reader = ReaderType::New();
    reader->SetFileName( argv[1] );
    typedef itk::VectorGradientAnisotropicDiffusionImageFilter< 
      InputImageType,InputImageType > FilterType;
    FilterType::Pointer smoother = FilterType::New();
    smoother->SetInput(reader->GetOutput());
    smoother->SetNumberOfIterations(   atoi( argv[4] ) );
    smoother->SetTimeStep( atof( argv[5] ) );
    try {
      smoother->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
    typedef itk::VectorPixel<unsigned char> WritePixelType;
    typedef itk::Image<WritePixelType,3> WriteImageType;
    typedef itk::VectorCastImageFilter<InputImageType,WriteImageType>CasterType;
    CasterType::Pointer caster = CasterType::New();
    typedef itk::ImageFileWriter<WriteImageType> WriterType;
    WriterType::Pointer writer = WriterType::New();
    caster->SetInput(smoother->GetOutput());
    writer->SetInput(caster->GetOutput());
    writer->SetFileName( argv[2] );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  return 0;
}
