#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkVectorGradientMagnitudeImageFilter.h"
#include "itkVectorGradientAnisotropicDiffusionImageFilter.h"
#include "itkVectorCastImageFilter.h"
#include "itkRGBPixel.h"
#include "itkWatershedImageFilter.h"
#include "itkCastImageFilter.h"
#include "itkGradientMagnitudeRecursiveGaussianImageFilter.h"
#include "itkUnaryFunctorImageFilter.h"
#include "itkScalarToRGBPixelFunctor.h"
#include "itkUnaryFunctorImageFilter.h"

int main( int argc, char * argv[] )
{
  if( argc < 6 ) {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile outputImageFile dimension lowerThreshold  outputScaleLevel" << std::endl;
    return 1;
  }
  
  // get the dimension from the user
  int dim = 2;
  if (atoi(argv[3]) == 3)
    dim = 3;
  
  typedef float pixelType;
  typedef itk::RGBPixel<unsigned char> RGBPixelType;
  
  if (dim == 2) {
    typedef itk::Image<pixelType, 2> InternalImageType;
    typedef itk::Image< RGBPixelType, 2>  RGBImageType;
    typedef  itk::ImageFileReader< InternalImageType > ReaderType;
    typedef  itk::ImageFileWriter< RGBImageType > WriterType;
    
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    
    typedef itk::GradientMagnitudeRecursiveGaussianImageFilter<
      InternalImageType,InternalImageType> GradientMagnitudeFilterType;
    GradientMagnitudeFilterType::Pointer gradientMagnitudeFilter = GradientMagnitudeFilterType::New();
    gradientMagnitudeFilter->SetInput( reader->GetOutput() );
    gradientMagnitudeFilter->SetSigma( 1.0 );
    typedef itk::WatershedImageFilter<InternalImageType> WatershedFilterType;
    
    WatershedFilterType::Pointer watershedFilter = WatershedFilterType::New();
    watershedFilter->SetInput( gradientMagnitudeFilter->GetOutput() );
    watershedFilter->SetThreshold( atof( argv[4] ) );
    watershedFilter->SetLevel(     atof( argv[5] ) );
    
    typedef itk::Functor::ScalarToRGBPixelFunctor<unsigned long> ColorMapFunctorType;
    typedef WatershedFilterType::OutputImageType  LabeledImageType;
    typedef itk::UnaryFunctorImageFilter<LabeledImageType,RGBImageType,ColorMapFunctorType
      > ColorMapFilterType;
    
    ColorMapFilterType::Pointer colorMapFilter = ColorMapFilterType::New();  
    colorMapFilter->SetInput(  watershedFilter->GetOutput() );
    
    writer->SetInput( colorMapFilter->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }

  else {
    typedef itk::Image<pixelType, 3> InternalImageType;
    typedef itk::Image< RGBPixelType, 3>  RGBImageType;
    typedef  itk::ImageFileReader< InternalImageType > ReaderType;
    typedef  itk::ImageFileWriter< RGBImageType > WriterType;
    
    ReaderType::Pointer reader = ReaderType::New();
    WriterType::Pointer writer = WriterType::New();
    
    reader->SetFileName( argv[1] );
    writer->SetFileName( argv[2] );
    
    typedef itk::GradientMagnitudeRecursiveGaussianImageFilter<
      InternalImageType,InternalImageType> GradientMagnitudeFilterType;
    GradientMagnitudeFilterType::Pointer gradientMagnitudeFilter = GradientMagnitudeFilterType::New();
    gradientMagnitudeFilter->SetInput( reader->GetOutput() );
    gradientMagnitudeFilter->SetSigma( 1.0 );
    typedef itk::WatershedImageFilter<InternalImageType> WatershedFilterType;
    
    WatershedFilterType::Pointer watershedFilter = WatershedFilterType::New();
    watershedFilter->SetInput( gradientMagnitudeFilter->GetOutput() );
    watershedFilter->SetThreshold( atof( argv[4] ) );
    watershedFilter->SetLevel(     atof( argv[5] ) );
    
    typedef itk::Functor::ScalarToRGBPixelFunctor<unsigned long> ColorMapFunctorType;
    typedef WatershedFilterType::OutputImageType  LabeledImageType;
    typedef itk::UnaryFunctorImageFilter<LabeledImageType,RGBImageType,ColorMapFunctorType
      > ColorMapFilterType;
    
    ColorMapFilterType::Pointer colorMapFilter = ColorMapFilterType::New();  
    colorMapFilter->SetInput(  watershedFilter->GetOutput() );
    
    writer->SetInput( colorMapFilter->GetOutput() );
    try {
      writer->Update();
    }
    catch( itk::ExceptionObject & excp ) {
      std::cerr << "Exception caught !" << std::endl;
      std::cerr << excp << std::endl;
    }
  }
  return 0;
}
